import gzip
import json

from locust import FastHttpUser, task

policy_rego = """
package app.abac

import rego.v1

default allow := false

allow if user_is_owner

allow if {
	user_is_employee
	action_is_read
}

allow if {
	user_is_employee
	user_is_senior
	action_is_update
}

allow if {
	user_is_customer
	action_is_read
	not pet_is_adopted
}

user_is_owner if data.user_attributes[input.user].title == "owner"

user_is_employee if data.user_attributes[input.user].title == "employee"

user_is_customer if data.user_attributes[input.user].title == "customer"

user_is_senior if data.user_attributes[input.user].tenure > 8

action_is_read if input.action == "read"

action_is_update if input.action == "update"

pet_is_adopted if data.pet_attributes[input.resource].adopted == true

"""

policy_data = """{"user_attributes":{"alice":{"tenure":20,"title":"owner"},"bob":{"tenure":15,"title":"employee"},"eve":{"tenure":5,"title":"employee"},"dave":{"tenure":5,"title":"customer"}},"pet_attributes":{"dog123":{"adopted":true,"age":2,"breed":"terrier","name":"toto"},"dog456":{"adopted":false,"age":3,"breed":"german-shepherd","name":"rintintin"},"dog789":{"adopted":false,"age":2,"breed":"collie","name":"lassie"},"cat123":{"adopted":false,"age":1,"breed":"fictitious","name":"cheshire"}}}"""
policy_input = {"user": "bob", "action": "read", "resource": "dog123"}


class PostUser(FastHttpUser):
    # Put up policy + data during Locust user instance start.
    def on_start(self):
        self.client.put("/v1/policies/abac_example", policy_rego)
        self.client.put(
            "/v1/data", data=policy_data, headers={"content-type": "application/json"}
        )

    # The actual query that Locust will hammer during the benchmark.
    @task
    def query_post(self):
        url = "/v1/data/app/abac"
        data = json.dumps(policy_input).encode("utf-8")
        headers = {"content-type": "application/json"}
        self.client.post(url, data=data, headers=headers)

    host = "http://localhost:8181"


class GzipPostUser(FastHttpUser):
    # Put up policy + data during Locust user instance start.
    def on_start(self):
        self.client.put("/v1/policies/abac_example", policy_rego)
        self.client.put(
            "/v1/data", data=policy_data, headers={"content-type": "application/json"}
        )

    # The actual query that Locust will hammer during the benchmark.
    @task
    def query_post(self):
        url = "/v1/data/app/abac"
        data = gzip.compress(json.dumps(policy_input).encode("utf-8"))
        headers = {"content-encoding": "gzip", "content-type": "application/json"}
        self.client.post(url, data=data, headers=headers)

    host = "http://localhost:8181"
